//---------------------------------------------------------------------------

#ifndef ListH
#define ListH
#include "system.hpp"
//---------------------------------------------------------------------------

struct Element {

	AnsiString		name;
	unsigned int    age;
	double			score;
	TDateTime		time;
	Element*		next;
	Element*		prev;

};

struct List {
	Element * first;
	Element * last;
};

/**
 * Initializes the first element in the list.
 */
void elem_init( List * list ) {

	list->first = 0;
	list->last  = 0;

}

/**
 * Adds an element to the end of the list.
 */
void elem_add( List * list, Element* elem ) {

	elem->next = 0;

	if (list->last) {
		list->last->next = elem;
		elem->prev 		 = list->last;
		list->last 		 = elem;
	} else {
		list->last 	= elem;
		list->first = elem;
		elem->prev 	= 0;
	}

}

/**
 * Inserts an element after another one.
 */
void elem_insert( List * list, Element* prev, Element* elem ) {

	elem->prev = prev;

	if (prev->next) {
		elem->next = prev->next;
		prev->next = elem;
	} else {
		elem->next = 0;
		prev->next = elem;
	}

	if (list->last == prev)
		list->last = elem;

}

/**
 * Removes an element from the list
 */
void elem_del( List * list, Element* elem ) {

	if (list->first == elem)
		list->first = elem->next;
	if (list->last == elem)
		list->last = elem->prev;

	elem->prev->next = elem->next;
	delete elem;
}

/**
 * Removes all elements from the list
 */
void elem_clear( List * list ) {

	while (list->last) {
		Element * el = list->last;
		list->last = el->prev;
		delete el;
	}

	list->first = 0;

}

/**
 * Fetches a list element by its index.
 */
Element* elem_byindex( List * list, unsigned int index ) {

	Element * elem = list->first;
	for (unsigned int i=0;i<index;i++) {
		elem = elem->next;
		if (!elem) return 0;
	}

	return elem;

}

/**
 * Finds an element with the given name.
 */
Element* elem_find_name( List * list, AnsiString name, bool search_up ) {

	Element * base;
	if (search_up) base = list->first;
	else base = list->last;

	while (base)
		if (base->name == name)
			return base;
		else
			if (search_up)
				base = base->next;
			else
				base = base->prev;

	return 0;
}

#endif
