#include <iostream>
#include <stdio.h>
#include <math.h>
#include "complex.h"
using namespace std;

ComplexNumber::ComplexNumber( double x, double y ) {

    this->x = x;
    this->y = y;

}

ComplexNumber ComplexNumber::conj() {

    return ComplexNumber( Re(), - Im() );
}

double ComplexNumber::abs() const {

    return sqrt( x*x + y*y );

}

double ComplexNumber::arg() const {

    return atan2( y, x );

}

ComplexNumber ComplexNumber::operator+( const ComplexNumber& z ) const {

    return ComplexNumber( x + z.Re(), y + z.Im() );
}

ComplexNumber operator+( double r, const ComplexNumber &z ) {

    return ComplexNumber( z.x + r, z.y );

}

ComplexNumber ComplexNumber::operator-( const ComplexNumber& z ) const {

    return ComplexNumber( x - z.Re(), y - z.Im() );
}

ComplexNumber operator-( const double r, const ComplexNumber &z )  {

    return ComplexNumber( z.x - r, z.y );
}

ComplexNumber ComplexNumber::operator-() const {

    return ComplexNumber( -x, -y );
}

ComplexNumber ComplexNumber::operator*( const ComplexNumber& z ) const {

    return ComplexNumber( x*z.Re() - y*z.Im(), x*z.Im() + y*z.Re() );
}

ComplexNumber operator*( const double r, const ComplexNumber &z ) {

    return ComplexNumber( z.x * r, z.y * r );
}

ComplexNumber ComplexNumber::operator/( const ComplexNumber& z ) const {

    double divisor = 1 / (z.Re()*z.Re() + z.Im()*z.Im());
    return ComplexNumber( 
        (x*z.Re() + y*z.Im()) * divisor, 
        (y*z.Re() - x*z.Im()) * divisor );
}

ComplexNumber operator/( const double r, const ComplexNumber &z ) {

    return ComplexNumber( z.x / r, z.y / r );
}

ComplexNumber& ComplexNumber::operator+=( const ComplexNumber& z ) {

    x += z.Re();
    y += z.Im();

    return *this;

}

ComplexNumber& ComplexNumber::operator-=( const ComplexNumber& z ) {

    x -= z.Re();
    y -= z.Im();

    return *this;

}

ComplexNumber& ComplexNumber::operator*=( const ComplexNumber& z ) {

    x = x*z.Re() - y*z.Im();
    y = x*z.Im() + y*z.Re();

    return *this;

}

ComplexNumber& ComplexNumber::operator/=( const ComplexNumber& z ) {

    double divisor = 1 / (z.Re()*z.Re() + z.Im()*z.Im());
    x = (x*z.Re() + y*z.Im()) * divisor;
    y = (y*z.Re() - x*z.Im()) * divisor;

    return *this;

}

ComplexNumber& ComplexNumber::operator+=( const double r ) {

    x += r;

    return *this;

}

ComplexNumber& ComplexNumber::operator-=( const double r ) {

    x -= r;

    return *this;

}

ComplexNumber& ComplexNumber::operator*=( const double r ) {

    x *= r;
    y *= r;

    return *this;

}

ComplexNumber& ComplexNumber::operator/=( const double r ) {

    x /= r;
    y /= r;

    return *this;

}

bool ComplexNumber::operator==( const ComplexNumber& z ) const {

    return ((x == z.Re()) && (y == z.Im()));

}

bool ComplexNumber::operator==( const double z ) const {

    return (x == z) && (y == 0);

}


std::string ComplexNumber::toString() const {

    char z[50];
    int b;
    std::string res;
    
    b = sprintf( z, "%.4f %c %.4fi", x, y<0?'-':'+', y<0?-y:y );

	res = ((string) z).substr(0,b);

	return res;

}


ostream& operator<<( ostream& os, const ComplexNumber &z ) {

	return os << z.toString();

}

ComplexNumber& ComplexNumber::operator++() {
	++x;
	return *this;
}
ComplexNumber& ComplexNumber::operator--() {
	--x;
	return *this;
}
ComplexNumber ComplexNumber::operator++(int) {
	++x;
	return *this;
}
ComplexNumber ComplexNumber::operator--(int) {
	--x;
	return *this;
}

bool ComplexNumber::operator<( const ComplexNumber &z ) const {
	return abs()<z.abs();
}
bool ComplexNumber::operator>( const ComplexNumber &z ) const {
	return abs()>z.abs();
}
bool ComplexNumber::operator<=( const ComplexNumber &z ) const {
	return abs()<=z.abs();
}
bool ComplexNumber::operator>=( const ComplexNumber &z ) const {
	return abs()>=z.abs();
}

