#include <string>
using namespace std;

/**
 * Represents a complex number z = x + iy
 */
class ComplexNumber {


    public:
	// constructors
	ComplexNumber( double x = 0, double y = 0);
	
	// accessor methods
	std::string toString() const;
	double Re() const { return x; }
	double Im() const { return y; }
	
	// operations
	ComplexNumber conj();
	double abs() const;
	double arg() const;
	
	// operator overloading
	ComplexNumber operator+( const ComplexNumber &z ) const;
	ComplexNumber operator-( const ComplexNumber &z ) const;
	ComplexNumber operator-() const;
	ComplexNumber operator*( const ComplexNumber &z ) const;
	ComplexNumber operator/( const ComplexNumber &z ) const;
	ComplexNumber& operator+=( const ComplexNumber &z );
	ComplexNumber& operator-=( const ComplexNumber &z );
	ComplexNumber& operator*=( const ComplexNumber &z );
	ComplexNumber& operator/=( const ComplexNumber &z );
	ComplexNumber& operator+=( const double r );
	ComplexNumber& operator-=( const double r );
	ComplexNumber& operator*=( const double r );
	ComplexNumber& operator/=( const double r );
	bool operator==( const ComplexNumber &z ) const;
	bool operator==( const double r ) const;
	friend ComplexNumber operator+( double r, const ComplexNumber &z );
	friend ComplexNumber operator-( double r, const ComplexNumber &z );
	friend ComplexNumber operator*( double r, const ComplexNumber &z );
	friend ComplexNumber operator/( double r, const ComplexNumber &z );

	ComplexNumber& operator++();
	ComplexNumber& operator--();
	ComplexNumber operator++(int);
	ComplexNumber operator--(int);

	bool operator<( const ComplexNumber &z ) const;
	bool operator>( const ComplexNumber &z ) const;
	bool operator<=( const ComplexNumber &z ) const;
	bool operator>=( const ComplexNumber &z ) const;

	private:
	double x, y;	// z = x + iy

};

/**
 * Overload ostream's << operator for the ComplexNumber class
 */
ostream& operator<<( ostream& os, const ComplexNumber &z );


