//---------------------------------------------------------------------------

#ifndef Unit2H
#define Unit2H
#include <Graphics.hpp>
//---------------------------------------------------------------------------

namespace grapp {

/**
 * Abstract base class for shapes.
 */
class Shape {

	protected:
	TColor color;
	unsigned int x0, y0;

	public:
	Shape( unsigned int x0 = 0, unsigned int y0 = 0, unsigned int color = 0 ):
		color(color),x0(x0),y0(y0) {}
	void setColor( TColor color ) { this->color = color; }
	void setOrigin( unsigned int x, unsigned int y ) { x0=x; y0=y; }
	void move( unsigned int x, unsigned int y ) { x0+=x; y0+=y; }
	void virtual draw( TCanvas &dc ) = 0;

};

class Rectangle : public Shape {

	unsigned int width, height;
	public:
	Rectangle( unsigned int x0 = 0, unsigned int y0 = 0, unsigned int width = 20,
		unsigned int height = 20 ):Shape(x0,y0),width(width),height(height) {}
	void draw( TCanvas &dc ) {
		dc.Pen->Color = color;
		dc.Brush->Color = color;
		dc.Rectangle( x0, y0, x0 + width, y0 + height );
	}
};

class Circle : public Shape {

	unsigned int radius;
	public:
	Circle( unsigned int x0 = 0, unsigned int y0 = 0, unsigned int radius = 20):
		Shape(x0,y0),radius(radius) {}
	void draw( TCanvas &dc ) {
		dc.Pen->Color = color;
		dc.Brush->Color = color;
		dc.Ellipse( x0, y0, x0+radius, y0+radius );
	}
};

class Triangle : public Shape {

	unsigned int side1;
	unsigned int side2;
	public:
	Triangle( unsigned int x0 = 0, unsigned int y0 = 0, unsigned int side1 = 20,
		unsigned int side2 = 20 ):
		Shape(x0,y0),side1(side1),side2(side2) {}
	void draw( TCanvas &dc ) {
		dc.Pen->Color = color;
		dc.Brush->Color = color;
		TPoint points[3];
		points[0] = Point(x0,y0);
		points[1] = Point(x0+side1,y0);
		points[2] = Point(x0,y0+side2);
		dc.Polygon( points, 2 );
	}
};

}
#endif
